@@ -1,5 +1,5 @@
 /* win-fns.h: Prototypes and declarations for win-fns.c
- * Copyright (C) 2005 Julian Graham
+ * Copyright (C) 2006 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,9 @@
 
 #include <libguile.h>
 
-SCM ruin_scm_api_window_focus_next(SCM id);
+SCM ruin_scm_api_window_focus_next(SCM);
+SCM ruin_scm_api_window_focus_prev(SCM);
+SCM ruin_scm_api_window_focus(SCM);
+SCM ruin_scm_api_window_render(SCM);
 
 #endif
