@@ -1,5 +1,5 @@
 /* modify.h: Prototypes and declarations for modify.c
- * Copyright (C) 2005 Julian Graham
+ * Copyright (C) 2006 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,5 +22,7 @@
 #include <libguile.h>
 
 SCM ruin_scm_default_handler_attr_modified(SCM event);
+SCM ruin_scm_default_handler_dom_node_inserted(SCM event);
+SCM ruin_scm_default_handler_dom_node_removed(SCM event);
 
 #endif
