@@ -1,5 +1,5 @@
 /* input.c: Input management routines for libRUIN
- * Copyright (C) 2005 Julian Graham
+ * Copyright (C) 2006 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -372,8 +372,8 @@ void ruin_input_send(ruin_window_t *w, int ch) {
 
   SCM key_loc = scm_c_eval_string("sdom:dom-key-location-standard");
 
-  ruin_util_log_current("dispatching event sdom:event-keydown for key '%s'\n",
-			SCM_STRING_CHARS(key_str));
+  ruin_util_log(w, "dispatching event sdom:event-keydown for key '%s'\n",
+		SCM_STRING_CHARS(key_str));
   
   scm_apply_0
     (scm_c_eval_string("sdom:dispatch-event"), 
@@ -381,8 +381,8 @@ void ruin_input_send(ruin_window_t *w, int ch) {
 		SCM_EOL, SCM_EOL, key_str, key_loc, SCM_BOOL_F, SCM_BOOL_F, 
 		SCM_BOOL_F, SCM_BOOL_F, SCM_UNDEFINED));
 
-  ruin_util_log_current("dispatching event sdom:event-keyup for key '%s'\n",
-			SCM_STRING_CHARS(key_str));
+  ruin_util_log(w, "dispatching event sdom:event-keyup for key '%s'\n",
+		SCM_STRING_CHARS(key_str));
 
   scm_apply_0
     (scm_c_eval_string("sdom:dispatch-event"), 
