@@ -130,8 +130,8 @@ ruin_element_t *ruin_element_new() {
 
   t->style_cache = ruin_util_hash_new();
 
-  t->inherent_attribute_style = scm_list(SCM_EOL);
-  t->additional_attribute_style = scm_list(SCM_EOL);
+  t->inherent_attribute_style = SCM_EOL;
+  t->additional_attribute_style = SCM_EOL;
   scm_gc_protect_object(t->inherent_attribute_style);
   scm_gc_protect_object(t->additional_attribute_style);
   t->current_pseudo_elements = ruin_util_list_new();
@@ -151,8 +151,7 @@ ruin_element_t *ruin_element_new() {
   t->margin_bottom.computed = RUIN_LAYOUT_VALUE_AUTO;
   t->margin_right.computed = RUIN_LAYOUT_VALUE_AUTO;
 
-  t->padding_left.computed = 1;
-  t->padding_right.computed = 1;
+  t->dirty = TRUE;
   return t;
 }
 
@@ -171,22 +170,6 @@ void ruin_element_free(ruin_element_t *t) {
   return;
 }
 
-void ruin_layout_append_child(ruin_element_t *p, ruin_element_t *c) {
-  if ((p == NULL) || (c == NULL))
-    return;
-  if (p->first_child == NULL)
-    p->first_child = c;
-  else {
-    ruin_element_t *elt_ptr = p->first_child;
-    while (elt_ptr->next_sibling != NULL)
-      elt_ptr = elt_ptr->next_sibling;
-    elt_ptr->next_sibling = c;
-    c->prev_sibling = elt_ptr;
-  }
-  c->parent = p;
-  return;
-}
-
 ruin_element_t *ruin_layout_find_containing_block(ruin_util_list *i, int dtm) {
   int j = 0, len = ruin_util_list_length(i);
   ruin_element_t *elt_ptr = NULL;
@@ -370,6 +353,9 @@ ruin_layout_size_t ruin_layout_size_text(ruin_element_t *t,
   return return_val;
 }
 
+/* Since the meaning of "containing block" can be a bit context-specific, we
+   use the display_type_mask to indicate what type of nodes we're counting as
+   containing blocks for resolving relative widths in this lookup... */
 
 void ruin_layout_normalize_length(ruin_window_t *w, ruin_length_t *l, 
 				  ruin_util_list *inheritance, int is_height,
@@ -565,10 +551,6 @@ ruin_layout_size_t ruin_layout_size_block(ruin_element_t *t,
      change after this point, so we shouldn't have to calculate this more than
      once. */
 
-  /* Briefly, set the used values for the vertical dimensions to their 
-     normalized computed values.  We should really make this do something
-     smarter. */
-    
   { ruin_element_t *elt_ptr;
     char *d = NULL;
     int offer_top = t->top + t->margin_top.used + t->border_top_width.used;
@@ -645,7 +627,7 @@ ruin_layout_size_t ruin_layout_size_table_fixed(ruin_element_t *t,
   ruin_element_t *row_ptr = NULL;
   int first_cell = TRUE;
 
-  ruin_layout_size_t return_val;
+  ruin_layout_size_t return_val = { 0, 0, 0, 0 };
   int offer_top = top, offer_left = left;
   int offer_top_adjusted = top;
 
@@ -716,7 +698,9 @@ ruin_layout_size_t ruin_layout_size_table_fixed(ruin_element_t *t,
 int ruin_layout_get_max_width(ruin_element_t *t, ruin_util_list *inheritance) {
   int max = 0;
   ruin_element_t *elt_ptr = t->first_child;
-  ruin_layout_normalize_lengths(t, inheritance, ruin_layout_block_mask);
+  int mask = ruin_layout_block_mask | RUIN_LAYOUT_DISPLAY_TABLE;
+  _ruin_layout_parse_sizes(t);
+  ruin_layout_normalize_lengths(t, inheritance, mask);
   while (elt_ptr != NULL) {
     if (strcmp(ruin_css_lookup(elt_ptr, "display"), "inline") != 0) {
       int r = ruin_layout_get_max_width(elt_ptr, inheritance);
@@ -737,6 +721,10 @@ int ruin_layout_get_max_width(ruin_element_t *t, ruin_util_list *inheritance) {
     }
     elt_ptr = elt_ptr->next_sibling;
   }
+
+  if (t->width.used > max)
+    max = t->width.used;
+
   if (t->padding_left.computed != RUIN_LAYOUT_VALUE_AUTO) 
     max += t->padding_left.used;
   if (t->padding_right.computed != RUIN_LAYOUT_VALUE_AUTO) 
@@ -754,13 +742,16 @@ int ruin_layout_get_max_width(ruin_element_t *t, ruin_util_list *inheritance) {
 int ruin_layout_get_min_width(ruin_element_t *t, ruin_util_list *inheritance) 
 {
   int max = 0;
-  int mask = ruin_layout_block_mask;
+  int mask = ruin_layout_block_mask | RUIN_LAYOUT_DISPLAY_TABLE;
   ruin_element_t *elt_ptr = NULL;
   char *d = ruin_css_lookup(t, "display");
+
+  _ruin_layout_parse_sizes(t);
+  ruin_layout_normalize_lengths(t, inheritance, mask);
+
   if (strcmp(d, "table-row") == 0) {   
     mask = RUIN_LAYOUT_DISPLAY_TABLE_ROW;
   } else if ((strcmp(d, "block") == 0) || (strcmp(d, "table-cell") == 0)) {
-    ruin_layout_normalize_lengths(t, inheritance, mask);
     elt_ptr = t->first_child;
     while (elt_ptr != NULL) {
       int width = 0;
@@ -787,6 +778,9 @@ int ruin_layout_get_min_width(ruin_element_t *t, ruin_util_list *inheritance)
     ruin_layout_normalize_lengths(t, inheritance, mask);
   }
   
+  if (t->width.used > max)
+    max = t->width.used;
+
   if (t->padding_left.computed != RUIN_LAYOUT_VALUE_AUTO) 
     max += t->padding_left.used;
   if (t->padding_right.computed != RUIN_LAYOUT_VALUE_AUTO) 
@@ -819,21 +813,24 @@ ruin_layout_size_t ruin_layout_size_table_cell(ruin_element_t *c,
   return result;
 }
 
+/* This function uses a layout algorithm derived from KHTML which is more or
+   less functionally equivalent to the one used by FF/IE. */
+
 ruin_layout_size_t ruin_layout_size_table_auto(ruin_element_t *t,
 					       ruin_util_list *inheritance,
 					       ruin_util_list *column_list,
 					       ruin_util_list *row_list,
 					       int top, int left) {
-  ruin_layout_size_t return_val;
-  ruin_util_list *column_max_widths = ruin_util_list_new();
-  ruin_util_list *column_min_widths = ruin_util_list_new();
+  ruin_layout_size_t return_val = { 0, 0, 0, 0 };
+  ruin_util_list *column_widths = ruin_util_list_new();
   int i = 0, num_rows = ruin_util_list_length(row_list);
   int num_columns = 0, longest_row = 0;
   int offer_top = top, offer_left = left, offer_top_adjusted;
-  int row_max_width = 0;
   int row_cumulative_width = 0;
 
   ruin_layout_normalize_lengths(t, inheritance, ruin_layout_block_mask);
+  t->width.used = 
+    _get_block_level_width(t, inheritance, ruin_layout_block_mask);
 
   return_val.height = 0;
   return_val.width = 0;
@@ -850,28 +847,58 @@ ruin_layout_size_t ruin_layout_size_table_auto(ruin_element_t *t,
     ruin_element_t *cell_ptr = row->first_child;
     ruin_layout_normalize_lengths
       (row, inheritance, RUIN_LAYOUT_DISPLAY_TABLE_ROW);
+
+    /* Obtain minimum and maximum content widths for the columns. */
+
     while(cell_ptr != NULL) {
       int max_width = 0, min_width = 0;
+      ruin_layout_table_width_blob_t *b = NULL;
+      ruin_element_t *width_calc_start = cell_ptr;
+      
+      /* If the table cell is an artificially inserted node (such as for XUL,
+	 which doesn't require anything like "td") we need to snarf the width
+	 from cell's first child. */
+
       _ruin_layout_parse_sizes(cell_ptr);
-      max_width = ruin_layout_get_max_width(cell_ptr, inheritance);
-      min_width = ruin_layout_get_min_width(cell_ptr, inheritance);
-      if (ruin_util_list_length(column_max_widths) < j + 1)
-	ruin_util_list_push_back(column_max_widths, 
-				 ruin_util_int_to_string(max_width));
-      else if (max_width > 
-	       atoi(ruin_util_list_peek_after(column_max_widths, j))) {
-	ruin_util_list_remove_after(column_max_widths, j);
-	ruin_util_list_insert_after(column_max_widths, j,
-				    ruin_util_int_to_string(max_width));
+      if (scm_string_p(cell_ptr->element) == SCM_BOOL_T) {
+	_ruin_layout_parse_sizes(cell_ptr->first_child);
+	cell_ptr->width = cell_ptr->first_child->width;
+	width_calc_start = cell_ptr->first_child;
       }
-      if (ruin_util_list_length(column_min_widths) < j + 1)
-	ruin_util_list_push_back(column_min_widths, 
-				 ruin_util_int_to_string(min_width));
-      else if (min_width > 
-	       atoi(ruin_util_list_peek_after(column_min_widths, j))) {
-	ruin_util_list_remove_after(column_min_widths, j);
-	ruin_util_list_insert_after(column_min_widths, j,
-				    ruin_util_int_to_string(min_width));
+
+      ruin_layout_normalize_lengths
+	(cell_ptr, inheritance, 
+	 ruin_layout_block_mask | RUIN_LAYOUT_DISPLAY_TABLE);
+
+      max_width = ruin_layout_get_max_width(width_calc_start, inheritance);
+      min_width = ruin_layout_get_min_width(width_calc_start, inheritance);
+
+      if (ruin_util_list_length(column_widths) < j + 1) {
+	b = calloc(1, sizeof(ruin_layout_table_width_blob_t));
+	b->max_width = max_width;
+	b->min_width = min_width;
+	b->width = cell_ptr->width;
+	ruin_util_list_push_back(column_widths, ruin_util_ptr_to_string(b));
+      }
+      else {
+	b = ruin_util_string_to_ptr
+	  (ruin_util_list_peek_after(column_widths, j));
+	if (max_width > b->max_width)
+	  b->max_width = max_width;
+	if (min_width > b->min_width)
+	  b->min_width = min_width;
+	if (cell_ptr->width.used != RUIN_LAYOUT_VALUE_AUTO) {
+
+	  /* Percentage widths apparently clobber other types of widths for the
+	     purposes of these calculations... */
+
+	  if (cell_ptr->width.units == RUIN_LAYOUT_UNITS_PERCENT)
+	    b->width = cell_ptr->width;
+	  else if (cell_ptr->width.used > b->width.used) {
+	      b->width.units = RUIN_LAYOUT_UNITS_CHARS;
+	      b->width.computed = b->width.used = cell_ptr->width.used;
+	  }
+	}
       }
       cell_ptr = cell_ptr->next_sibling;
       j++;
@@ -884,7 +911,7 @@ ruin_layout_size_t ruin_layout_size_table_auto(ruin_element_t *t,
      columns! */
 
   /* There may not be as many columns specified by the user as there are cells
-     in a row; we'll need to add them as necessary. */
+     in a row; we'll need to add them here as necessary. */
 
   { ruin_element_t *new_column_group = ruin_element_new();
     ruin_element_t *sib_ptr = NULL;
@@ -922,38 +949,168 @@ ruin_layout_size_t ruin_layout_size_table_auto(ruin_element_t *t,
 
   num_columns = longest_row;
 
-  if (t->width.computed != RUIN_LAYOUT_VALUE_AUTO) {
-    
-    /* If the used width for the table is greater than the sum of the minimum
-       widths, then distribute the extra over the columns; but the columns get
-       the minimum in this case. */
+  /* Now set the effective widths on the columns. */
+  /* TODO: Revise this and the above to take spanning cells into account. */
 
-    int column_sum = 0, leftovers = 0;
-    for (i = 0; i < num_columns; i++)
-      column_sum += atoi(ruin_util_list_peek_after(column_min_widths, i));
-    leftovers = t->width.used - column_sum;
-    while (leftovers > 0) {
-      for (i = 0; (leftovers > 0) && (i < num_columns); leftovers--, i++) {
-	int new_val = atoi(ruin_util_list_peek_after(column_min_widths, i));
-	ruin_util_list_remove_after(column_min_widths, i);
-	ruin_util_list_insert_after
-	  (column_min_widths, i, ruin_util_int_to_string(new_val));
+  { int p_widths = 0, abs_mwidths = 0, auto_mwidths = 0; 
+    int num_auto = 0;
+    int available = t->width.used;
+    for (i = 0; i < num_columns; i++) {
+      ruin_layout_table_width_blob_t *b = 
+	(ruin_layout_table_width_blob_t *) ruin_util_string_to_ptr
+	(ruin_util_list_peek_after(column_widths, i));
+      ruin_element_t *col = (ruin_element_t *) ruin_util_string_to_ptr
+	(ruin_util_list_peek_after(column_list, i));
+      b->eff_min_width = b->min_width;
+      b->eff_max_width = b->max_width;
+      if (b->width.units == RUIN_LAYOUT_UNITS_PERCENT)
+	p_widths += b->width.computed;
+      else if (b->width.computed == RUIN_LAYOUT_VALUE_AUTO) {
+	auto_mwidths += b->eff_max_width;
+	num_auto++;
+      }
+      else abs_mwidths += b->eff_max_width;
+      col->width = b->eff_width = b->width;
+    }
+
+    /* Distribute width to percent-valued columns. */
+
+    if ((available > 0) && (p_widths > 0)) {
+      for (i = 0; i < num_columns; i++) {
+	ruin_element_t *col = (ruin_element_t *) ruin_util_string_to_ptr
+	  (ruin_util_list_peek_after(column_list, i));
+	if (col->width.units == RUIN_LAYOUT_UNITS_PERCENT) {
+	  ruin_layout_table_width_blob_t *b =
+	    (ruin_layout_table_width_blob_t *) ruin_util_string_to_ptr
+	    (ruin_util_list_peek_after(column_widths, i));
+	  int w = MIN(b->eff_min_width, col->width.used);
+	  col->width.used = w;
+	  available -= w;
+	}
+      }
+      if (p_widths > 100) {
+	/* Reduce, starting from the right, the percent column widths 'til
+	   they're under 100... */
       }
     }
-  }
 
-  for (i = 0; i < num_columns; i++) {
-    ruin_element_t *col_ptr = (ruin_element_t *) 
-      ruin_util_string_to_ptr(ruin_util_list_peek_after(column_list, i));
-    if (col_ptr->width.computed != RUIN_LAYOUT_VALUE_AUTO) {
-      
-      /* What do we do here? */
-      
-    } else {
-      col_ptr->width.used = 
-	atoi(ruin_util_list_peek_after(column_min_widths, i));
-      row_cumulative_width += col_ptr->width.used;
-    }      
+    /* Adjust fixed-width columns if effective width is more than computed
+       width. */
+    
+    if (available > 0) {
+      for (i = 0; i < num_columns; i++) {
+	ruin_element_t *col = (ruin_element_t *) ruin_util_string_to_ptr
+	  (ruin_util_list_peek_after(column_list, i));
+	if ((col->width.units != RUIN_LAYOUT_UNITS_PERCENT) && 
+	    (col->width.computed != RUIN_LAYOUT_VALUE_AUTO)) {
+	  ruin_layout_table_width_blob_t *b =
+	    (ruin_layout_table_width_blob_t *) ruin_util_string_to_ptr
+	    (ruin_util_list_peek_after(column_widths, i));
+	  int w = MAX(b->eff_width.used, col->width.used);
+	  col->width.used = w;
+	  available -= w;
+	}
+      }
+    }
+
+    /* Distribute the remaining width among auto-valued columns */
+
+    if ((available > 0) && (num_auto > 0)) {
+      for (i = 0; i < num_columns; i++) {
+	ruin_element_t *col = (ruin_element_t *) ruin_util_string_to_ptr
+	  (ruin_util_list_peek_after(column_list, i));
+	if (col->width.computed == RUIN_LAYOUT_VALUE_AUTO) {
+	  ruin_layout_table_width_blob_t *b =
+	    (ruin_layout_table_width_blob_t *) ruin_util_string_to_ptr
+	    (ruin_util_list_peek_after(column_widths, i));
+	  int w = b->eff_max_width * (available / num_auto);
+	  col->width.used = w;
+	  available -= w;
+	}
+      }
+    }
+
+    /* If there's still available width, add it to percentage and fixed-width
+       columns. */
+    
+    if ((available > 0) && (p_widths > 0)) {
+      for (i = 0; i < num_columns; i++) {
+	ruin_element_t *col = (ruin_element_t *) ruin_util_string_to_ptr
+	  (ruin_util_list_peek_after(column_list, i));
+	if (col->width.units == RUIN_LAYOUT_UNITS_PERCENT) {
+	  ruin_layout_table_width_blob_t *b =
+	    (ruin_layout_table_width_blob_t *) ruin_util_string_to_ptr
+	    (ruin_util_list_peek_after(column_widths, i));
+	  int w = b->eff_max_width * (available / p_widths);
+	  col->width.used += w;
+	  available -= w;
+	}
+      }
+    }
+    if ((available > 0) && (abs_mwidths > 0)) {
+      for (i = 0; i < num_columns; i++) {
+	ruin_element_t *col = (ruin_element_t *) ruin_util_string_to_ptr
+	  (ruin_util_list_peek_after(column_list, i));
+	if (col->width.computed != RUIN_LAYOUT_VALUE_AUTO) {
+	  ruin_layout_table_width_blob_t *b =
+	    (ruin_layout_table_width_blob_t *) ruin_util_string_to_ptr
+	    (ruin_util_list_peek_after(column_widths, i));
+	  int w = b->eff_max_width * (available / abs_mwidths);
+	  col->width.used += w;
+	  available -= w;
+	}
+      }
+    }
+    
+    /* If we've run out of available width, we need to trim some from the
+       columns we've allocated it to. */
+
+    if (available < 0) {
+      int able_to_reduce;
+      do {
+	able_to_reduce = FALSE;
+	for (i = 0; i < num_columns; i++) {
+	  ruin_element_t *col = (ruin_element_t *) ruin_util_string_to_ptr
+	    (ruin_util_list_peek_after(column_list, i));
+	  if ((col->width.computed == RUIN_LAYOUT_VALUE_AUTO) &&
+	      (col->width.used > 0)) {
+	    col->width.used--;
+	    able_to_reduce = TRUE;
+	  }
+	} 
+      } while ((available < 0) && able_to_reduce);      
+    }
+    if (available < 0) {
+      int able_to_reduce;
+      do {
+	able_to_reduce = FALSE;
+	for (i = 0; i < num_columns; i++) {
+	  ruin_element_t *col = (ruin_element_t *) ruin_util_string_to_ptr
+	    (ruin_util_list_peek_after(column_list, i));
+	  if ((col->width.units != RUIN_LAYOUT_UNITS_PERCENT) &&
+	      (col->width.computed != RUIN_LAYOUT_VALUE_AUTO) &&
+	      (col->width.used > 0)) {
+	    col->width.used--;
+	    able_to_reduce = TRUE;
+	  }
+	}
+      } while ((available < 0) && able_to_reduce);      
+    }
+    if (available < 0) {
+      int able_to_reduce;
+      do {
+	able_to_reduce = FALSE;
+	for (i = 0; i < num_columns; i++) {
+	  ruin_element_t *col = (ruin_element_t *) ruin_util_string_to_ptr
+	    (ruin_util_list_peek_after(column_list, i));
+	  if ((col->width.units == RUIN_LAYOUT_UNITS_PERCENT) &&
+	      (col->width.used > 0)) {
+	    col->width.used--;
+	    able_to_reduce = TRUE;
+	  }
+	}
+      } while ((available < 0) && able_to_reduce);      
+    }
   }
 
   offer_top_adjusted = offer_top;
@@ -970,13 +1127,13 @@ ruin_layout_size_t ruin_layout_size_table_auto(ruin_element_t *t,
     int offer_left_adjusted = offer_left;
     int row_max_height = 0;
 
-    offer_top_adjusted += t->padding_top.used;
-    t->height.used += t->padding_top.used;
+    offer_top_adjusted += t->padding_top.used + t->border_top_width.used;
+    t->height.used += t->padding_top.used + t->border_top_width.used;
 
     while (cell_ptr != NULL) {
       ruin_layout_size_t result;
-      offer_left_adjusted += t->padding_left.used;
-      row_cumulative_width += t->padding_left.used;
+      offer_left_adjusted += t->padding_left.used + t->border_left_width.used;
+      row_cumulative_width += t->padding_left.used + t->border_left_width.used;
 
       /* This is wrong as it stands... */
 
@@ -1001,23 +1158,18 @@ ruin_layout_size_t ruin_layout_size_table_auto(ruin_element_t *t,
       offer_left_adjusted += col_ptr->width.used;
 
       cell_ptr = cell_ptr->next_sibling;
+      row_cumulative_width += col_ptr->width.used;
       col_ptr = col_ptr->next_sibling;
 
       j++;
     }
 
-    /* if ((row->width.computed != RUIN_LAYOUT_VALUE_AUTO) &&
-	(row->width.used > row_cumulative_width))
-      row_cumulative_width = row->width.used;
-      else row->width.used = row_cumulative_width;
-
-    if (row_cumulative_width > row_max_width)
-    row_max_width = row_cumulative_width; */
-
     row->width.used = row_cumulative_width;
 
-    offer_top_adjusted += row_max_height + t->padding_bottom.used + 1;
-    t->height.used += row_max_height + t->padding_bottom.used + 1;
+    offer_top_adjusted += row_max_height + t->padding_bottom.used + 
+      t->border_bottom_width.used + 2;
+    t->height.used += row_max_height + t->padding_bottom.used + 
+      t->border_bottom_width.used + 2;
     
     /* Update the row-group's height as well. */
     
@@ -1026,16 +1178,10 @@ ruin_layout_size_t ruin_layout_size_table_auto(ruin_element_t *t,
     if (row_cumulative_width > row->parent->width.used) {
       row->parent->width.used = row_cumulative_width;
     }
-    row->parent->height.used += row_max_height + t->padding_bottom.used + 1;
+    row->parent->height.used += row_max_height + t->padding_bottom.used + 
+      t->border_bottom_width.used + 1;
   }  
 
-  if (t->width.used > row_max_width) {
-
-    /* Distribute the extra width across the columns... */
-
-    if (t->width.computed != RUIN_LAYOUT_VALUE_AUTO) { }
-  } else t->width.used = row_max_width;
-
   return_val.height = t->height.used;
   return_val.width = t->width.used;
 
@@ -1171,12 +1317,6 @@ ruin_layout_size_t ruin_layout_size_list_item(ruin_element_t *t,
     t->border_top_width.used + t->border_bottom_width.used;
   return_val.width = tentative_width;
 
-
-  /* return_val.width = t->margin_left.used + t->margin_right.used + 
-    t->border_left_width.used + t->border_right_width.used + 
-    (2 * t->padding_left.used) + t->padding_right.used + marker_len + 
-    max_child_width; */
-
   ruin_util_list_pop_front(inheritance); 
   return return_val;
 }
@@ -1188,16 +1328,9 @@ ruin_layout_size_t ruin_layout_size_none(ruin_element_t *t,
   /* This isn't exactly how this should work... */
 
   ruin_layout_size_t nil = { 0, 0, 0, 0 };
-
-  /*  if (t->next_sibling != NULL)
-    return ruin_layout_size_tree(t->next_sibling);
-    else */
-  
   return nil;
 }
 
-/* The 'm' parameter should always be the width of the containing block... */
-
 ruin_layout_size_t ruin_layout_size_tree(ruin_element_t *t,
 					 ruin_util_list *inheritance,
 					 int top, int left) {
@@ -1221,7 +1354,7 @@ ruin_layout_size_t ruin_layout_size_tree(ruin_element_t *t,
 void ruin_layout_add_style(SCM *list, char *prop, char *value) {
   SCM addition = scm_list_2(scm_makfrom0str(prop), scm_makfrom0str(value));
   if (scm_eq_p(*list, SCM_EOL) != SCM_BOOL_T)
-    scm_append_x(scm_list_2(SCM_CADR(*list), addition));
+    scm_append_x(scm_list_2(SCM_CDR(*list), scm_list_1(addition)));
   else {
     *list = scm_list_2(scm_list_1(scm_makfrom0str("*")), addition);
     scm_gc_protect_object(*list);
