@@ -17,7 +17,6 @@
 
 (define-module (sdom core)
   #:use-module (ice-9 regex)
-  #:use-module (ice-9 slib)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-13)
   #:use-module (sxml ssax)
@@ -704,24 +703,23 @@
 	((7) (string-copy (cadr node)))
 	((3) "#text")))))
 
-(define derive-value (lambda (n) (derive-valuet n (sdom:node-type n))))
-
-(define derive-valuet
-  (lambda (node type)
-    (case type
-      ((2) (if (sdom:get-dom-property node "sdom:specified")
-	       (if (sdom:has-child-nodes? node)
-		   (sdom:get-dom-property (sdom:get-dom-property 
-					   node "sdom:first-child")
-					  "sdom:node-value") "")))
-      
-      ;; HAVE TO HANDLE ENTITY-REFS, LOOKUPS, ETC. !!!
-      
-      ((4) (car node))
-      ((3) (car node))
-      ((8) (cadr node))
-      ((7) (caddr node))
-      (else '()))))
+(define derive-value
+  (lambda (node)
+    (let ((type (sdom:node-type node)))
+      (case type
+	((2)
+	 (if (sdom:get-dom-property node "sdom:specified")
+	     (if (sdom:has-child-nodes? node)
+		 (sdom:get-dom-property (sdom:get-dom-property 
+					 node "sdom:first-child")
+					"sdom:node-value") "")))
+	
+	;;		 '())) ;; HAVE TO HANDLE ENTITY-REFS, LOOKUPS, ETC. !!!
+	((4) (car node))
+	((3) (car node))
+	((8) (cadr node))
+	((7) (caddr node))
+	(else '())))))
 
 (define set-value!
   (lambda (node value)
@@ -818,12 +816,15 @@
 
 (define extract-attributes
   (lambda (node)
-    (let ((node-list (single-at-finder node)))
-      (if (not (eq? node-list #f))
-	  (map (lambda (item) (cons '@ item))
-	       (filter (lambda (item) (not (eq? (car item) '@)))
-		       (cdr node-list)))
-	  '()))))
+    (let ((type (sdom:node-type node)))
+      (cond ((eqv? type sdom:node-type-element)
+	     (let ((node-list (single-at-finder node)))
+	       (if (not (eq? node-list #f))
+		   (map (lambda (item) (cons '@ item))
+			(filter (lambda (item) (not (eq? (car item) '@)))
+				(cdr node-list)))
+		   '())))
+	    (else '())))))
 
 (define extract-children
   (lambda (node)
@@ -1002,7 +1003,7 @@
 				 ,sdom:node-type-cdata-section
 				 ,sdom:node-type-comment
 				 ,sdom:node-type-processing-instruction))
-		    (derive-valuet x type))
+		    (derive-value x))
 		   (else '()))))
 	,(lambda (x y)
 	   (let ((children (extract-children x))
@@ -1030,12 +1031,9 @@
     (,sdom:node-type-character-data 
      (@ (sdom:data ,derive-value ,set-value!)
 	(sdom:length ,(lambda (node) 
-			   (let ((v (derive-valuet 
-				     node sdom:node-type-character-data)))
+			   (let ((v (derive-value node)))
 			     (if (not (null? v))
-				 (string-length 
-				  (derive-valuet
-				   node sdom:node-type-character-data))
+				 (string-length (derive-value node))
 				 0)))))
      (,sdom:node-type-text 
       (@ (sdom:is-element-content-whitespace #f)
@@ -2127,9 +2125,11 @@
 ;;                                                                           ;;
 ;;---------------------------------------------------------------------------;;
 
-(define internal-get-attribute-node
-  (lambda (elt name ns)
-    (let* ((finder (if (null? ns) 
+(define sdom:get-attribute-node
+  (lambda (elt name . namespace-uri) 
+    (if (not (eqv? (sdom:node-type elt) sdom:node-type-element))
+	(throw 'sdom:exception sdom:exception-code-type-mismatch-err))
+    (let* ((finder (if (null? namespace-uri) 
 		       (lambda (x) (equal? name (sdom:get-dom-property 
 						 x "sdom:name")))
 		       (lambda (x) (and (equal? name 
@@ -2137,21 +2137,10 @@
 						 x "sdom:local-name"))
 					(equal? (sdom:get-dom-property 
 						 x "sdom:namespace-uri")
-						ns)))))
+						(car namespace-uri))))))
 	   (match (find finder (extract-attributes elt))))
       (if match match '()))))
 
-(define sdom:get-attribute-node
-  (lambda (elt name . namespace-uri) 
-    (if (not (eqv? (sdom:node-type elt) sdom:node-type-element))
-	(throw 'sdom:exception sdom:exception-code-type-mismatch-err))
-    (if (null? namespace-uri) 
-	(internal-get-attribute-node elt name '())
-	(let ((cns (car namespace-uri)))
-	  (if (not (string? cns))
-	      (throw 'sdom:exception sdom:exception-code-type-mismatch-err)
-	      (internal-get-attribute-node elt name cns))))))
-
 (define sdom:set-attribute-node-internal!
   (lambda (elt node)
     (let ((x (find (lambda (y) (equal? (sdom:get-dom-property node "sdom:name")
@@ -2199,9 +2188,9 @@
 (define sdom:get-attribute
   (lambda (elt name . namespace-uri)
     (let ((node (if (not (null? namespace-uri))
-		    (internal-get-attribute-node elt name (car namespace-uri))
-		    (internal-get-attribute-node elt name '()))))
-      (if (null? node) '() (derive-valuet node sdom:node-type-attr)))))
+		    (sdom:get-attribute-node elt name (car namespace-uri))
+		    (sdom:get-attribute-node elt name))))
+      (if (null? node) '() (derive-value node)))))
 
 (define sdom:set-attribute!
   (lambda (elt name value . namespace-uri)
@@ -2243,8 +2232,7 @@
 			     sdom:node-type-element)
 		       (find (lambda (y) 
 			       (and (sdom:get-dom-property y "sdom:is-id")
-				    (equal? id (derive-valuet 
-						y sdom:node-type-attr))))
+				    (equal? id (derive-value y))))
 			     (extract-attributes x))))))
       (if (not (eqv? (sdom:node-type doc) sdom:node-type-document))
 	  (throw 'sdom:exception sdom:exception-code-type-mismatch-err))
