@@ -1,5 +1,5 @@
 /* util.c: Utility functions for libRUIN
- * Copyright (C) 2005 Julian Graham
+ * Copyright (C) 2006 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -317,38 +317,6 @@ void ruin_util_hash_free(ruin_util_hash *hash) {
   return;
 }
 
-/* ruin_util_hash *ruin_property_string_to_hash(char *properties) {
-  char *name = NULL;
-  int i, properties_length;
-  int ws_current = FALSE, name_current = FALSE, value_current = FALSE;
-  int chunk_start = -1;
-  ruin_util_hash *return_hash;
-  if (properties == NULL)
-    return NULL;
-  properties_length = strlen(properties);
-  return_hash = ruin_util_hash_new();
-  for (i = 0; i < properties_length; i++) {
-    if (i == 0) {
-      if (isspace(properties[i])) ws_current = TRUE;
-      else if (isalpha(properties[i])) name_current = TRUE;
-      else break;
-    }
-    if (isspace(properties[i])) {
-      if (!value_current) break; 
-    }
-    else if (properties[i] == '\"') {
-      if (!value_current) break;
-    }
-    else if (properties[i] == '=') {
-      if (name_current) {
-	name = malloc(i - chunk_start + 1, sizeof(char));
-      }
-    }
-    else if (properties[i] == '\\') {
-    }
-  }
-  } */
-
 /* Here are the list functions. */
 
 ruin_util_list *ruin_util_list_new() {
@@ -620,18 +588,13 @@ long ruin_util_generate_id() {
   return return_val;
 }
 
-void ruin_util_log_current(char *msg, ...) {
-  extern ruin_windows_t *_ruin_windows;
+void ruin_util_log(ruin_window_t *w, char *msg, ...) {
   va_list args;
-  int window_id = scm_num2int(scm_c_eval_string("ruin:current-window"), 0, "");
-  ruin_window_t *win = (ruin_window_t *) ruin_util_string_to_ptr
-    (ruin_util_hash_retrieve(_ruin_windows->windows, 
-			     ruin_util_int_to_string(window_id)));
   va_start(args, msg);
-  if (win->log != NULL) {
-    fprintf(win->log, "window %d: ", window_id);
-    vfprintf(win->log, msg, args);
-    fflush(win->log); 
+  if (w->log != NULL) {
+    fprintf(w->log, "window %d: ", w->internal_id);
+    vfprintf(w->log, msg, args);
+    fflush(w->log); 
   }
   va_end(args);
 }
