@@ -165,16 +165,16 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
     }
   } else if (strcmp((char *) attr_name, "align") == 0) {
     if (strcmp(attr_str, "start") == 0) {
-      ruin_layout_add_style(&t->inherent_attribute_style, "align", "left");
+      ruin_layout_add_style(&t->additional_attribute_style, "align", "left");
       ruin_layout_add_style
-	(&t->inherent_attribute_style, "vertical-align", "top");
+	(&t->additional_attribute_style, "vertical-align", "top");
     } else if (strcmp(attr_str, "center") == 0) {
-      ruin_layout_add_style(&t->inherent_attribute_style, "align", "center");
+      ruin_layout_add_style(&t->additional_attribute_style, "align", "center");
       ruin_layout_add_style
-	(&t->inherent_attribute_style, "vertical-align", "center");
+	(&t->additional_attribute_style, "vertical-align", "center");
     } else if (strcmp(attr_str, "end") == 0) {
-      ruin_layout_add_style(&t->inherent_attribute_style, "align", "right");
-      ruin_layout_add_style(&t->inherent_attribute_style, "vertical-align",
+      ruin_layout_add_style(&t->additional_attribute_style, "align", "right");
+      ruin_layout_add_style(&t->additional_attribute_style, "vertical-align",
 			    "bottom");
     }
   } else if (strcmp((char *) attr_name, "allowevents") == 0) {
@@ -194,7 +194,7 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
   } else if (strcmp((char *) attr_name, "flags") == 0) {
   } else if (strcmp((char *) attr_name, "flex") == 0) {
   } else if (strcmp((char *) attr_name, "height") == 0) {
-    int height = 0;
+    /*    int height = 0;
     int attr_str_len = strlen((char *) attr_str);
     if (sscanf((char *) attr_str, "%d", &height) == 1) {
       t->height.computed = height;
@@ -202,7 +202,8 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
       if ((attr_str_len > 1) && 
 	  (strcmp(attr_str + attr_str_len - 1, "%") == 0))
 	t->height.units = RUIN_LAYOUT_UNITS_PERCENT;
-    }
+	} */
+    ruin_layout_add_style(&t->additional_attribute_style, "height", attr_str);
   } else if (strcmp((char *) attr_name, "hidden") == 0) {
   } else if (strcmp((char *) attr_name, "id") == 0) {
     if (strlen(attr_str) > 0) {
@@ -242,9 +243,11 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
   } else if (strcmp((char *) attr_name, "ordinal") == 0) {
   } else if (strcmp((char *) attr_name, "orient") == 0) {
     if (strcmp(attr_str, "horizontal") == 0) {
-      ruin_layout_add_style(&t->inherent_attribute_style, "display", "table");
+      ruin_layout_add_style
+	(&t->additional_attribute_style, "display", "table");
     } else if (strcmp(attr_str, "vertical") == 0) {
-      ruin_layout_add_style(&t->inherent_attribute_style, "display", "block");
+      ruin_layout_add_style
+	(&t->additional_attribute_style, "display", "block");
     }
   } else if (strcmp((char *) attr_name, "pack") == 0) {
   } else if (strcmp((char *) attr_name, "persist") == 0) {
@@ -264,7 +267,7 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
   } else if (strcmp((char *) attr_name, "uri") == 0) {
   } else if (strcmp((char *) attr_name, "wait-cursor") == 0) {
   } else if (strcmp((char *) attr_name, "width") == 0) {
-    int width = 0;
+    /* int width = 0;
     int attr_str_len = strlen((char *) attr_str);
     if (sscanf((char *) attr_str, "%d", &width) == 1) {
       t->width.computed = width;
@@ -272,7 +275,8 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
       if ((attr_str_len > 1) && 
 	  (strcmp(attr_str + attr_str_len - 1, "%") == 0))
 	t->width.units = RUIN_LAYOUT_UNITS_PERCENT;
-    }
+    } */
+    ruin_layout_add_style(&t->additional_attribute_style, "width", attr_str);
   }
 
   /* These attributes are specific to certain elements. */
@@ -297,27 +301,8 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
     if ((strcmp((char *) parent_name, "button") == 0) ||
 	(strcmp((char *) parent_name, "caption") == 0) ||
 	(strcmp((char *) parent_name, "menuitem") == 0) ||
-	(strcmp((char *) parent_name, "radio") == 0)) {
-      /* This is a hack. */
-
-      ruin_element_t *child = ruin_element_new();
-
-      child->dialect = t->dialect;
-      child->doc = t->doc;
-      child->element = scm_makfrom0str("libruin-block-element-label");
-      scm_gc_protect_object(child->element);
-      child->cascade = t->cascade;
-
-      ruin_layout_add_style
-	(&child->inherent_attribute_style, "text-align", "center");
-      ruin_layout_add_style
-	(&child->inherent_attribute_style, "display", "inline");
-      ruin_layout_add_style
-	(&child->inherent_attribute_style, "color", 
-	 ruin_css_lookup(t, "color"));
-      child->content = strdup(attr_str);
-      ruin_layout_append_child(t, child);
-    }
+	(strcmp((char *) parent_name, "radio") == 0)) 
+      t->first_child = ruin_dialect_generate_text_node(node, t, NULL);
   } else if (strcmp((char *) attr_name, "open") == 0) {
   } else if (strcmp((char *) attr_name, "orient") == 0) {    
   } else if (strcmp((char *) attr_name, "selected") == 0) {
@@ -443,12 +428,12 @@ ruin_element_t *ruin_xul_generate_tree(ruin_window_t *w, SCM node,
       my_result->focusable = TRUE;
       my_result->extra_content = RUIN_LAYOUT_EXTRA_CONTENT_CHECKBOX;
     }
-    else if (strcmp((char *) node_name, "caption") == 0) {
+    /* else if (strcmp((char *) node_name, "caption") == 0) {
       if ((parent != NULL) && (parent->caption == NULL)) {
 	parent->caption = my_result;	
 	not_sibling = TRUE;
       }
-    }
+      } */
     else if (strcmp((char *) node_name, "colorpicker") == 0) {
       my_result->focusable = TRUE;
       my_result->extra_content = RUIN_LAYOUT_EXTRA_CONTENT_COLORPICKER;
